package org.rcisoft.business.memGoldCoinFlow.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.rcisoft.business.memGoldCoinFlow.dao.MemGoldCoinFlowRepository;
import org.rcisoft.business.memGoldCoinFlow.entity.MemGoldCoinFlow;
import org.rcisoft.business.memGoldCoinFlow.service.MemGoldCoinFlowService;
import org.rcisoft.business.memInfo.dao.MemInfoRepository;
import org.rcisoft.business.memInfo.entity.MemInfo;
import org.rcisoft.core.util.CyUserUtil;
import org.rcisoft.core.model.CyPersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.rcisoft.core.model.CyPageInfo;

import java.util.List;
import lombok.extern.slf4j.Slf4j;

/**
 * Created by cy on 2025年2月7日 上午10:11:22.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class MemGoldCoinFlowServiceImpl extends ServiceImpl<MemGoldCoinFlowRepository, MemGoldCoinFlow> implements MemGoldCoinFlowService {

    @Autowired
    private MemInfoRepository memInfoRepository;

    /**
    * 保存 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel persist(MemGoldCoinFlow memGoldCoinFlow){
        //增加到金币流水表
        memGoldCoinFlow.setEndCount(memGoldCoinFlow.getCount() + memGoldCoinFlow.getBalance());
        int line = baseMapper.insert(memGoldCoinFlow);
        //修改会员表的金币余额
        MemInfo memInfo = new MemInfo();
        memInfo.setBusinessId(memGoldCoinFlow.getMemberId());
        memInfo.setGoldCoinsCount(memGoldCoinFlow.getCount() + memGoldCoinFlow.getBalance());
        memInfoRepository.updateById(memInfo);
        //增加订单信息
        return new CyPersistModel(line);
    }

    /**
    * 删除 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel remove(MemGoldCoinFlow memGoldCoinFlow){
        int line = baseMapper.realDelete(memGoldCoinFlow);
        log.debug(CyUserUtil.getAuthenUsername()+"删除了ID为"+
            memGoldCoinFlow.getBusinessId()+"的金币流水表信息");
        return new CyPersistModel(line);
    }

    /**
    * 逻辑删除 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel removeLogical(MemGoldCoinFlow memGoldCoinFlow){
        memGoldCoinFlow.setDeleted();
        int line = baseMapper.deleteById(memGoldCoinFlow);
            log.debug(CyUserUtil.getAuthenUsername()+"逻辑删除了ID为"+
                memGoldCoinFlow.getBusinessId()+"的金币流水表信息");
            return new CyPersistModel(line);
    }

    /**
    * 修改 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public CyPersistModel merge(MemGoldCoinFlow memGoldCoinFlow){
        int line = baseMapper.updateById(memGoldCoinFlow);
        log.debug(CyUserUtil.getAuthenUsername()+"修改了ID为"+ memGoldCoinFlow.getBusinessId()+"的金币流水表信息");
        return new CyPersistModel(line);
    }

    /**
    * 根据id查询 金币流水表
    * @param id
    * @return
    */
    @Override
    public MemGoldCoinFlow findById(int id){
        return baseMapper.selectById(id);
    }

    /**
    * 分页查询 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Override
    public IPage<MemGoldCoinFlow> findAllByPagination(CyPageInfo<MemGoldCoinFlow> paginationUtility,
        MemGoldCoinFlow memGoldCoinFlow){
        return baseMapper.queryMemGoldCoinFlowsPaged(paginationUtility,memGoldCoinFlow);

    }

    /**
    * 查询list 金币流水表
    * @param memGoldCoinFlow
    * @return
    */
    @Override
    public List<MemGoldCoinFlow> findAll(MemGoldCoinFlow memGoldCoinFlow){
        return baseMapper.queryMemGoldCoinFlows(memGoldCoinFlow);

    }

    /**
    * 导出金币流水表
    * @return
    */
    @Override
    public List<MemGoldCoinFlow> export(MemGoldCoinFlow memGoldCoinFlow) {
        List<MemGoldCoinFlow> memGoldCoinFlowList = baseMapper.queryMemGoldCoinFlows(memGoldCoinFlow);
        return memGoldCoinFlowList;
    }


    /**
     * 查询当前用户金币余额
     */
    @Override
    public Integer balance() {
        Integer userId = Integer.valueOf(CyUserUtil.getAuthenBusinessId());
        Integer balance = baseMapper.balance(userId);
        return balance;
    }
}
